#include "main.h"
#include "arguments.h"
#include "verbose.h"

void eusage(char * name, int exit_flag)
{
  printf("Usage: %s [OPTIONS]... [PORT]\n", name);
  if (exit_flag) exit(1);
}

void usage(char * name)
{
  eusage(name, 1);
}

void help(char * name)
{
  eusage(name, 0);
  printf("\n");
  printf("Mandatory arguments to long options are mandatory for short options too.\n");
  printf("  -h, --help                 shows this help\n");
  printf("  -c, --config               sets configuration file\n");
  printf("  -v, --verbose              enables verbose output\n");
  printf("\n");
  exit(0);
}

void parse_options(int argc, char * const argv[])
{
  static struct option long_options[] = 
    {
      {"verbose", no_argument,       0, 'v'},
      {"help",    no_argument,       0, 'h'},
      {"limit",   required_argument, 0, 'l'},
      {0, 0, 0, 0}
    };

  int c;

  verbose_flag = 0;
  while (1) {
    int option_index = 0;

    c = getopt_long(argc, argv, "vhl:", long_options, &option_index);

    if (c == -1)
      break;

    switch (c) {
      case 'v':
        verbose_flag++;
        break;
      case 'h':
        help(argv[0]); 
        break;
      case 'l':
        limit = atoi(optarg);
        break;
      case '?':
        usage(argv[0]);
        break;
      default:  
        abort();
    }
  }

  if (optind + 1 == argc) {
    port = argv[optind++];
  }

  /* Print any remaining command line arguments (not options). */
  if (optind < argc) {
    printf ("non-option ARGV-elements: ");
    while (optind < argc)
      printf ("%s ", argv[optind++]);
    putchar ('\n');
  }

  /* Instead of reporting `--verbose'
   * and `--brief' as they are encountered,
   * we report the final status resulting from them. */
  Vprintf1("Verbose mode enabled at level %d\n", verbose_flag);
  Vprintf1("limit = %d\n", limit);
  Vprintf1("port = \"%s\"\n", port);

}

