#include "main.h"
#include "serial.h"
#include "rs232.h"
#include "handlers.h"
#include "limit.h"
#include "verbose.h"

// define handlers
struct message_handler handlers[] =
{
  {CMD_RESP_VOL_CHANGE, vol_message},
  {CMD_RESP_MODEL_INFO, model_message},
  {CMD_RESP_SPEAKER_A,  speaker_message},
  {CMD_RESP_SPEAKER_B,  speaker_message},
  {CMD_REQ_GET_TIME,  get_time_message},
  {CMD_REQ_SET_TIME,  set_time_message},
  {0, 0}
};

void model_message(HANDLER_ARGS)
{
  int i;
  printf("Connected to \"");
  for (i = 3; i <= data[0]; i++) {
    printf("%c", data[i]);
  }
  printf("\"\n");
  sendcmd(&nad_fdi, CMD_REQ_VOL);
  req_speaker_a();
  req_speaker_b();
}

void vol_message(HANDLER_ARGS)
{
  signed char vol = data[3];
  printf("volume = %d dB\n", vol);

  if (vol > limit) {
    printf("volume (%d dB) over limit (%d dB)\n", vol, limit);
    printf("enforcing limit\n");
    set_volume(limit);
  }
}

void speaker_message(HANDLER_ARGS)
{
  UNCH speaker = data[2] - '\x3A';
  UNCH value = data[3];
  printf("speaker %X = %d\n", ((UNCH) speaker) + 10, value);

  if (value == 1 && speakers[speaker] == 0) {
    printf("speaker %X is ON but should be OFF\n", ((UNCH) speaker) + 10);
    printf("enforcing limit\n");
    speaker_helper(1, (UNCH) speaker, 0);
  }
}

void get_time_message(HANDLER_ARGS)
{
  struct timeval tv;
  struct timezone tz;
  gettimeofday(&tv, &tz);
  char buf[7];
  buf[0] = 0x06;
  buf[1] = 0x4C;
  buf[2] = 0x20;
  memcpy(&buf[3], &tv.tv_sec, 4);
  sendcmd(fdi, buf);
}

void set_time_message(HANDLER_ARGS)
{
  // TODO: checkauth()
  Vprintf3("Time change request\n");
  if (data[0] != 6) return;
  struct timeval tv;
  struct timezone tz;
  gettimeofday(&tv, &tz);
  memcpy(&tv.tv_sec, &data[3], 4);
  settimeofday(&tv, &tz);
}

