#include "main.h"
#include "rs232.h"
#include "limit.h"
#include "handlers.h"

void speaker_helper(UNCH set, UNCH speaker, UNCH flag)
{
  char buffer[5];
  bzero(buffer, 5);
  buffer[0] = set ? 3 : 2;
  buffer[1] = 20 + (UNCH) set;
  buffer[2] = 58 + (UNCH) speaker;
  buffer[3] = (UNCH) flag;
  sendcmd(&nad_fdi, buffer); 
}

void set_speaker_a(UNCH flag)
{
  speaker_helper(1, 0, flag);
}

void set_speaker_b(UNCH flag)
{
  speaker_helper(1, 1, flag);
}

void req_speaker_a()
{
  speaker_helper(0, 0, 0);
}

void req_speaker_b()
{
  speaker_helper(0, 1, 0);
}

void set_volume(int vol)
{
  char buffer[5];
  bzero(buffer, 5);
  buffer[0] = 3;
  buffer[1] = 21;
  buffer[2] = 23;
  buffer[3] = (signed char) vol;
  sendcmd(&nad_fdi, buffer); 
}

void change_limit(int nl)
{
  if (limit != nl) {
    limit = nl;
    sendcmd(&nad_fdi, CMD_REQ_VOL);
  }
}

void change_limit_speakers(UNCH i, UNCH v)
{
  if (speakers[i] != v) {
    printf("Limit for speaker %X changed!\n", 10+i);
    speakers[i] = v;
    speaker_helper(0, i, 0);
  }
}

void adjustlimit()
{
  static struct config_entry config[] = {
    // {day, hour, maxvol, {allow A, allow B}}}

    {1,  2, -10,  {1, 1}},

    {1,  8,   5,  {1, 1}},
    {1, 23,  -5,  {1, 1}},
    {2,  2, -10,  {1, 1}},

    {2,  8,   5,  {1, 1}},
    {2, 23,  -5,  {1, 1}},
    {3,  2, -10,  {1, 1}},

    {3,  8,   5,  {1, 1}},
    {3, 23,  -5,  {1, 1}},
    {4,  2, -10,  {1, 1}},

    {4,  8,   5,  {1, 1}},
    {4, 23,  -5,  {1, 1}},
    {5,  2, -10,  {1, 1}},

    {5,  8,   5,  {1, 1}},
    {6,  2,  -5,  {1, 1}},

    {6,  8,   5,  {1, 1}},
    {7,  2,  -5,  {1, 1}},

    {7,  8,   5,  {1, 1}},
    {7, 23,  -5,  {1, 1}},

    {0,  0,   0,  {0, 0}}
  };
  static signed char config_cur = -1;
 
  time_t s =  time(NULL);
  struct tm curtm;
  localtime_r(&s, &curtm);
 
  UNCH max_config = 0;
  unsigned short int max_hours = 0;

  // find the current configuration
  UNCH i;
  for (i = 0; config[i].wday; i++) {
    signed short int diff_hours;
    diff_hours = config[i].hour - curtm.tm_hour - 1;
    diff_hours += 24 * ((config[i].wday == 7 ? 0 : config[i].wday) - curtm.tm_wday);

    if (diff_hours < 0) {
      diff_hours += 7 * 24;
    }

    if (diff_hours > max_hours) {
      max_hours = diff_hours;
      max_config = i;
    }
  }

  // change configuration, if diffrent
  if (config_cur != max_config) {
    config_cur = max_config;

    printf("Switching to new config:\n");
    printf("  Limit: %d dB\n", config[config_cur].limit);
    for (i=0;i<2;i++) {
      printf("  Speaker %X %s!\n", 10+i, config[config_cur].speaker[i] ? "may be ON" : "shall be OFF at all times");
    }

    change_limit(config[config_cur].limit);
    change_limit_speakers(0, config[config_cur].speaker[0]);
    change_limit_speakers(1, config[config_cur].speaker[1]);
  }
}

