#include "main.h"
#include "arguments.h"
#include "handlers.h"
#include "serial.h"
#include "socket.h"
#include "rs232.h"
#include "limit.h"
#include "verbose.h"

void idleloop()
{
  Vprintf("Initiating idle loop\n");

  /* Do the select */
  while (1) {
    /* Initialize the input set */
    fd_set input;
    FD_ZERO(&input);
    FD_SET(nad_fdi.fd, &input);

    //int max_fd = (sock > fd ? sock : fd);
    int max_fd = nad_fdi.fd;

    // add the socket, and its connections to the set
    socket_build_select_list(&input, &max_fd);
    
    Vprintf3("max_fd: %d\n", max_fd);

    /* Initialize the timeout structure */
    struct timeval timeout;
    timeout.tv_sec  = 60;
    timeout.tv_usec = 0;

    // call select(2)
    int n = select(max_fd+1, &input, NULL, NULL, &timeout);
    Vprintf3("We have run select\n");

    /* See if there was an error */
    if (n < 0) {
      perror("select failed");
    } else if (n == 0) {
      Vprintf(".");
    } else {
      /* We have input */
      Vprintf3("We have input\n");
      if (FD_ISSET(nad_fdi.fd, &input)) {
        Vprintf3("hi");
        processinput(&nad_fdi);
      }

      // handle socket events
      socket_read_selected(&input);
    }
    
    adjustlimit();
  }
}

int main(int argc, char * const argv[])
{
  // handle config
  limit = DEFAULT_LIMIT;
  port = DEFAULT_PORT;
  speakers[0] = 1;
  speakers[1] = 1;
  parse_options(argc, argv);

  // open the port
  open_port();

  // adjust limit
  adjustlimit();

  // request model info
  sendcmd(&nad_fdi, CMD_REQ_MODEL);
  //sendcmd(CMD_REQ_ALL); // ALL includes model

  // open up the pipe
  socket_open();

  // initiate idle loop
  idleloop();

  // clean up
  close_sockets(); // close socket connections
  close_port(); // close RS232 port

  Vprintf("Exiting...\n");
  return 0;
}
