#include "main.h"
#include "handlers.h"
#include "serial.h"
#include "socket.h"
#include "rs232.h"
#include "verbose.h"

char chksum(char * data)
{
  int i;
  UNCH foo = '\x00';
  Vprintf3("chksum:");
  for (i=1;i<=data[0];i++) {
    Vprintf3(" %02X", (UNCH) data[i]);
    foo += data[i];
  }
  foo = 0xff - foo;
  Vprintf3(" = %02X\n", foo);
  return foo;
}

void sendcmd(fdinfo * fdi, char * data)
{
  if (data[0] < 1) return;

  UNCH sum = chksum(data);
  UNCH buffer[15];
  int i = 0;
  int j = 0;
  buffer[i++] = '\x01';
  for (j = 1; j <= data[0]; j++) {
    if ((UNCH) data[j] < '\20') {
      buffer[i++] = '\x5E';
      buffer[i++] = data[j] | '\x40';
    } else if (data[j] == '\x5E') {
      buffer[i++] = '\x5E';
      buffer[i++] = '\x5E';
    } else {
      buffer[i++] = data[j];
    }
  }
  buffer[i++] = '\x02';
  buffer[i++] = sum;
  
  Vprintf("Sending: ");
  for (j=0;j<i;j++) Vprintf(" %02X",buffer[j]);
  Vprintf(" to fd %d\n", fdi->fd);

  ssize_t ret = 0;
  if (fdi->type == FD_TYPE_CONN) {
    ret = write(fdi->fd, buffer, i);
    if (ret == -1) {
      perror("write");
    }
  } else {
    ssize_t cnt = 0;
    while (cnt != i) {
      ret = write(fdi->fd, buffer, i);
      if (ret > 0) {
        cnt += ret;
      }
      if (ret == -1 && errno != EAGAIN) {
        perror("write");
      }
    }
  }
}

int readcmd(fdinfo * fdi, char * cmd)
{
  char buffer[CMDBUFLEN];
  unsigned short int buffer_pos = 0;
  char cbuffer[CMDBUFLEN];
  ssize_t cbytes;
  UNCH record = 0;
  bzero(buffer, 15);

  while (1) {

    cbytes = read(fdi->fd, cbuffer, 1);

    if (cbytes == -1) {
      if (errno == EAGAIN) {
        if (record == 0) break;
        continue;
      }

      usleep(500000);

      // TODO: use edge triggering and buffer the rest

      perror("read");

      if (fdi->type == FD_TYPE_CONN) {
        close_socket_by_fdi(fdi);
      }
      break;
    }
    if (cbytes == 0) {
      if (fdi->type == FD_TYPE_CONN) {
        Vprintf3("Lost connection\n");
        close_socket_by_fdi(fdi);
        break;
      }
    }
    //printf("%d, %02X\n", cbytes, (UNCH) cbuffer[0]);
    switch (cbuffer[0]) {
      case '\x01':
        record = 1;
        break;
      case '\x02':
        if (record == 1) {
          record = 2;
        }
        break;
      default:
        if (record == 1) {
          buffer[buffer_pos++] = cbuffer[0];
        } else if (record == 2) {
          Vprintf("Input:");
          int i; for (i=0;i<strlen(buffer);i++) Vprintf(" %02X", (UNCH) buffer[i]);
          Vprintf("\n");
          
          // decode;
          bzero(cmd, 15);
          int j = 1;
          for (i = 0; i < strlen(buffer); i++) {
            if (buffer[i] == '\x5E' && i+1 < strlen(buffer)) {
              i++;
              if (buffer[i] == '\x5E') {
                cmd[j++] = buffer[i];
              } else {
                cmd[j++] = buffer[i] & 191;
              }
            } else {
              cmd[j++] = buffer[i];
            }
          }
          cmd[0] = j - 1;
          
          Vprintf("CMD:");
          for (i=1;i<=cmd[0];i++) Vprintf(" %02X", (UNCH) cmd[i]);
          Vprintf(" = %X\n", cbuffer[0]);

          if ((UNCH) chksum(cmd) == (UNCH) cbuffer[0]) {
            return strlen(cmd);
          } else {
            Vprintf("chksum: %02X != %02X\n", (UNCH) chksum(cmd), (UNCH) cbuffer[0]);
            return -1;
          }
        }
        break;
    }
  }
  return 0;
}

void processinput(fdinfo * fdi)
{
  char response[CMDBUFLEN];
  int i, j;
  int found = 0;
  Vprintf3("fd: %d\n", fdi->fd);
  readcmd(fdi, response);
  if (fdi->fd == 0) return;
  Vprintf("Looking for handler to:");
  int k; for (k=1;k<=response[0];k++) Vprintf(" %02X", (UNCH) response[k]);
  Vprintf("\n");
  for (i = 0; handlers[i].trigger && !found; i++) {
    if (response[0] < handlers[i].trigger[0])
      continue;
    for (j = 1; j <= handlers[i].trigger[0] + 1; j++) {
      if (handlers[i].trigger[0] == j-1) {
        Vprintf3("Calling handler for:");
        int l; for (l=1;l<j;l++) Vprintf3(" %02X", (UNCH) response[l]);
        Vprintf3("\n");
        handlers[i].handler(fdi, response);
        found = 1;
        break;
      } else if (handlers[i].trigger[j] != response[j]) {
        break;
      }
    }
  }
  
  if (fdi->type == FD_TYPE_RS232) {
    Vprintf3("Forwarding from serial to network.\n");
    socket_broadcast(response);
  }
  
  if (fdi->type == FD_TYPE_CONN && response[1] != 'L') {
    Vprintf3("Forwarding from network to serial.\n");
    sendcmd(&nad_fdi, response);
  }
}

