#include "main.h"
#include "serial.h"

/* Serial port */
char * port;

fdinfo nad_fdi; /* File descriptor information for the port */

/*
 * 'open_port()'
 *
 * Returns the file descriptor on success or -1 on error.
 */
void open_port()
{
  nad_fdi.type = FD_TYPE_RS232;
  nad_fdi.fd = open(port, O_RDWR | O_NOCTTY | O_NDELAY);
  if (nad_fdi.fd == -1)
  {
    /*
     * Could not open the port.
     */

    char buf[60];
    snprintf(buf, 60, "open_port: Unable to open port %s", port);
    perror(buf);
    abort();
  }

  // set nonblocking
  fcntl(nad_fdi.fd, F_SETFL, FNDELAY);

  // set blocking
  //fcntl(fd, F_SETFL, 0);

  // get current options
  struct termios options;
  tcgetattr(nad_fdi.fd, &options);

  // No parity (8N1)
  options.c_cflag &= ~PARENB;
  options.c_cflag &= ~CSTOPB;
  options.c_cflag &= ~CSIZE;
  options.c_cflag |= CS8;
  
  // raw input mode
  options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

  // raw output mode
  options.c_oflag &= ~OPOST;

  // disable hardware flow control
  options.c_cflag &= ~CRTSCTS;

  // disable software flow control
  options.c_iflag &= ~(IXON | IXOFF | IXANY);

  // set baud
  cfsetispeed(&options, BAUDRATE);
  cfsetospeed(&options, BAUDRATE);

  // set options
  tcsetattr(nad_fdi.fd, TCSANOW, &options);
}

void close_port()
{
  AZ(close(nad_fdi.fd));
}

